from typing import IO, List

import click
import requests

from treebeard.conf import secrets_endpoint
from treebeard.helper import get_treebeard_env, validate_notebook_directory

from treebeard.secrets.helper import get_secrets_archive

treebeard_env = get_treebeard_env()


@click.group()
def secrets():
    pass


@secrets.command()
@click.argument("files", type=click.File("r"), nargs=-1)
def push(files: List[IO]):
    """Uploads json document of secrets to Treebeard's servers. These will be available in your notebook directory at runtime"""
    click.echo(f"🌲 Pushing Secrets for project {treebeard_env['project_id']}")
    validate_notebook_directory()
    secrets_archive = get_secrets_archive(files)
    response = requests.post(
        secrets_endpoint,
        files={"secrets": open(secrets_archive.name, "rb")},
        headers=treebeard_env,
    )
    if response.status_code != 200:
        click.echo(
            f"Error: service failure pushing secrets, {response.status_code}: {response.text}"
        )
        return

    click.echo("🔐  done!")



import click

from treebeard.helper import env, get_treebeard_env

if env == "development":
    click.echo("WARNING: RUNNING IN LOCAL MODE")
    url = "http://localhost:8080"
    treebeard_web_url = "http://localhost:8000"
else:
    url = "https://scheduler-cvee2224cq-ew.a.run.app"
    treebeard_web_url = "https://treebeard.io"


treebeard_env = get_treebeard_env()
project_id = treebeard_env["project_id"]
notebook_id = treebeard_env["notebook_id"]
secrets_endpoint = f"{url}/projects/{project_id}/notebooks/{notebook_id}/secrets"
notebooks_endpoint = f"{url}/notebooks/{notebook_id}"
signup_endpoint = f"{url}/cli_signup"
service_status_endpoint = f"{url}/service_status"



import glob
import json
import os
import os.path
import sys
import tarfile
import tempfile
import time
from datetime import datetime
from typing import Any, List
import docker  # type: ignore

import click
import requests
from dateutil import parser
from halo import Halo  # type: ignore
from humanfriendly import format_size, parse_size  # type: ignore
from timeago import format as timeago_format  # type: ignore

from treebeard.conf import notebooks_endpoint
from treebeard.helper import (
    get_config_path,
    get_treebeard_env,
    get_time,
    validate_notebook_directory,
    get_treebeard_config,
)
from treebeard.secrets.helper import get_secrets_archive

config_path = get_config_path()

treebeard_env = get_treebeard_env()
notebook_id = treebeard_env["notebook_id"]
project_id = treebeard_env["project_id"]


@click.command()
@click.option("t", "--hourly", flag_value="hourly", help="Run notebook hourly")
@click.option("t", "--daily", flag_value="daily", help="Run notebook daily")
@click.option("t", "--weekly", flag_value="weekly", help="Run notebook weekly")
@click.option(
    "t", "--quarter-hourly", flag_value="quarter-hourly", help="Run quarter-hourly"
)
@click.option(
    "watch", "--watch", is_flag=True, help="Run and check completed build status"
)
@click.option(
    "local", "--local", is_flag=True, help="Build image with local docker installation",
)
@click.option(
    "-i",
    "--ignore",
    help="Don't submit unneeded virtual envs and large files",
    multiple=True,
)
def run(t: str, watch: bool, ignore: List[str], local: bool):
    """
    Run a notebook and optionally schedule it to run periodically
    """

    validate_notebook_directory()

    params = {}
    if t:
        params["schedule"] = t

    spinner: Any = Halo(text="🌲  Compressing Repo\n", spinner="dots")
    spinner.start()

    treebeard_config = get_treebeard_config()
    if treebeard_config:
        ignore += (
            treebeard_config.ignore
            + treebeard_config.secret
            + treebeard_config.output_dirs
        )

    # Create a temporary file for the compressed directory
    # compressed file accessible at f.name
    # git_files: Set[str] = set(
    #     subprocess.check_output(
    #         "git ls-files || exit 0", shell=True, stderr=subprocess.DEVNULL
    #     )
    #     .decode()
    #     .splitlines()
    # )

    with tempfile.NamedTemporaryFile(
        "wb", suffix=".tar.gz", delete=False
    ) as src_archive:
        click.echo("\n")
        with tarfile.open(fileobj=src_archive, mode="w:gz") as tar:

            def zip_filter(info: tarfile.TarInfo):
                for ignored in ignore:
                    if info.name in glob.glob(ignored):
                        return None

                # if len(git_files) > 0 and info.name not in git_files:
                #     return None
                click.echo(f"  Including {info.name}")
                return info

            tar.add(
                os.getcwd(), arcname=os.path.basename(os.path.sep), filter=zip_filter
            )
            tar.add(config_path, arcname=os.path.basename(config_path))
    size = os.path.getsize(src_archive.name)
    max_upload_size = "100MB"
    if size > parse_size(max_upload_size):
        click.echo(
            click.style(
                (
                    f"ERROR: Compressed notebook directory is {format_size(size)},"
                    f" max upload size is {max_upload_size}. \nPlease ensure you ignore any virtualenv subdirectory"
                    " using `treebeard run --ignore venv`"
                ),
                fg="red",
            )
        )
        quit(1)

    if local:
        spinner.stop()
        spinner: Any = Halo(text="🌲  Compressing Secrets\n", spinner="dots")
        spinner.start()
        # os.chdir(tempfile.gettempdir())
        client: Any = docker.from_env()
        builder_image_name = "gcr.io/treebeard-259315/image-builder"
        client.images.pull(builder_image_name)
        secrets_archive = get_secrets_archive()

        build_tag = str(time.mktime(datetime.today().timetuple()))
        env_vars = {
            "TREEBEARD_PROJECT_ID": project_id,
            "TREEBEARD_NOTEBOOK_ID": notebook_id,
            "TREEBEARD_REPO_URL": f"file://{src_archive.name}",
            "TREEBEARD_SECRETS_URL": f"file://{secrets_archive.name}",
            "TREEBEARD_BUILD_TAG": build_tag,
        }

        spinner.text = "🌲  Building Image Locally\n"
        docker_socket = "/var/run/docker.sock"
        repo_image_name = (
            f"gcr.io/treebeard-259315/projects/{project_id}/{notebook_id}:{build_tag}"
        )
        click.echo(f"Building {repo_image_name}")

        click.echo(f"Env vars: {env_vars}")
        # exit(0)
        build_logs = client.containers.run(
            builder_image_name,
            "/repo/run_build",
            environment=env_vars,
            volumes={
                src_archive.name: {"bind": src_archive.name, "mode": "ro"},
                secrets_archive.name: {"bind": secrets_archive.name, "mode": "ro"},
                docker_socket: {"bind": docker_socket, "mode": "rw"},
            },
            stdout=True,
            stderr=True,
            detach=False,
        )
        click.echo(build_logs)
        click.echo("Running locally")
        run_logs = client.containers.run(
            repo_image_name,
            "/home/project_user/execute_notebook",
            environment={
                "TREEBEARD_PROJECT_ID": project_id,
                "TREEBEARD_NOTEBOOK_ID": notebook_id,
                "GOOGLE_APPLICATION_CREDENTIALS": "/tmp/keys/google_creds.json",
            },
            volumes={
                docker_socket: {"bind": docker_socket, "mode": "rw"},
                os.getenv("GOOGLE_APPLICATION_CREDENTIALS"): {
                    "bind": "/tmp/keys/google_creds.json",
                    "mode": "ro",
                },
            },
            stdout=True,
            stderr=True,
            detach=False,
        )
        click.echo(run_logs)
        sys.exit(0)

    spinner.text = "🌲  submitting notebook to runner\n"
    response = requests.post(
        notebooks_endpoint,
        files={"repo": open(src_archive.name, "rb")},
        params=params,
        headers=treebeard_env,
    )

    if response.status_code != 200:
        raise click.ClickException(f"Request failed: {response.text}")

    spinner.stop()
    try:
        json_data = json.loads(response.text)
        click.echo(f"✨  Run has been accepted! {json_data['admin_url']}")
    except:
        click.echo("❗  Request to run failed")
        click.echo(sys.exc_info())

    if watch:
        # spinner = Halo(text='watching build', spinner='dots')
        # spinner.start()
        build_result = None
        while not build_result:
            time.sleep(5)
            response = requests.get(notebooks_endpoint, headers=treebeard_env)
            json_data = json.loads(response.text)
            status = json_data["runs"][-1]["status"]
            click.echo(f"{get_time()} Build status: {status}")
            if status == "SUCCESS":
                build_result = status
                # spinner.stop()
                click.echo(f"Build result: {build_result}")
            elif status in ["FAILURE", "TIMEOUT", "INTERNAL_ERROR", "CANCELLED"]:
                click.echo(f"Build failed")
                build_result = status
                sys.exit(1)


@click.command()
def cancel():
    """Cancels the current notebook build and schedule"""
    validate_notebook_directory()
    spinner: Any = Halo(text="cancelling", spinner="dots")
    click.echo(f"🌲  Cancelling {notebook_id}")
    spinner.start()
    requests.delete(notebooks_endpoint, headers=treebeard_env)  # type: ignore
    spinner.stop()
    click.echo(f"🛑 Done!")


@click.command()
def status():
    """Show the status of the current notebook"""
    validate_notebook_directory()
    response = requests.get(notebooks_endpoint, headers=treebeard_env)
    if response.status_code != 200:
        raise click.ClickException(f"Request failed: {response.text}")

    json_data = json.loads(response.text)
    if len(json_data) == 0:
        click.echo(
            "This notebook has not been run. Try running it with `treebeard run`"
        )
        quit(1)
    click.echo("🌲  Recent runs:\n")

    max_results = 10
    status_emoji = {
        "SUCCESS": "✅",
        "QUEUED": "💤",
        "WORKING": "⏳",
        "FAILURE": "❌",
        "TIMEOUT": "⏰",
        "CANCELLED": "🛑",
    }

    for run in json_data["runs"][-max_results:]:
        now = parser.isoparse(datetime.utcnow().isoformat() + "Z")
        start_time = parser.isoparse(run["start_time"])
        time_string: str = timeago_format(start_time, now=now)
        click.echo(f"{status_emoji[run['status']]}  {time_string} {run['url']}")

    if "schedule" in json_data:
        click.echo(f"\n📅  Schedule: {json_data['schedule']}")



import warnings
import webbrowser

import click
from halo import Halo  # type: ignore
from humanfriendly import format_size, parse_size  # type: ignore
from timeago import format as timeago_format  # type: ignore

from treebeard.conf import (
    signup_endpoint,
    treebeard_web_url,
)
from treebeard.helper import (
    get_config_path,
    get_treebeard_env,
    set_credentials,
)
from treebeard.version import get_version

config_path = get_config_path()

treebeard_env = get_treebeard_env()

project_id = treebeard_env["project_id"]
notebook_id = treebeard_env["notebook_id"]

warnings.filterwarnings(
    "ignore", "Your application has authenticated using end user credentials"
)


@click.command()
@click.option("--email", prompt="Your email:")
@click.option("--key", prompt="Your API key:")
def configure(email: str, key: str):
    """Set initial credentials"""
    project_id = set_credentials(email, key, signup_endpoint)
    webbrowser.open_new_tab(
        f"{treebeard_web_url}/cli_signup?email={email}&api_key={key}&project_id={project_id}"
    )


@click.command()
def version():
    """Shows treebeard package version"""
    click.echo(get_version())


@click.command()
def info():
    """Shows treebeard credentials and project info"""
    click.echo(treebeard_env)



from treebeard.helper import get_run_path, get_treebeard_config, TreebeardConfig
from google.cloud import storage  # type: ignore
import papermill as pm  # type: ignore
import threading
from datetime import datetime
import os
from typing import Any, Callable

storage_client: Any = storage.Client()
treebeard_config = get_treebeard_config() or TreebeardConfig()
bucket_name = "treebeard-notebook-outputs"
output_notebook_local_path = "/tmp/out.ipynb"
run_path = get_run_path()
global cancelled
cancelled = False


def log(message: str):
    print(f'{datetime.now().strftime("%H:%M:%S")}: {message}')


def set_interval(func: Callable, sec: int):
    def func_wrapper():
        if cancelled:
            return
        func()
        set_interval(func, sec)

    t = threading.Timer(sec, func_wrapper)
    t.start()
    return t


def cancel_interval():
    log("Cancelling Interval")
    global cancelled
    cancelled = True


def save_artifacts():
    log(f"saving all artifacts")

    notebook_upload_path = f"{run_path}/out.ipynb"
    if os.path.exists(output_notebook_local_path):
        log(f"Saving {output_notebook_local_path} to {notebook_upload_path}")
        notebook_blob = storage_client.bucket(bucket_name).blob(notebook_upload_path)
        notebook_blob.upload_from_filename("/tmp/out.ipynb")
    else:
        log("No output notebook to save")

    for output_dir in treebeard_config.output_dirs:
        for root, _, files in os.walk(output_dir, topdown=False):
            for name in files:
                full_name = os.path.join(root, name)
                upload_path = f"{run_path}/{full_name}"
                log(f"Saving {full_name} to {upload_path}")
                blob = storage_client.bucket(bucket_name).blob(upload_path)
                blob.upload_from_filename(full_name)


def run():
    for output_dir in treebeard_config.output_dirs:
        os.makedirs(output_dir, exist_ok=True)
    set_interval(save_artifacts, 10)
    try:
        path, notebook_name = os.path.split(treebeard_config.notebook)
        log(f"Executing Notebook {notebook_name} in {path}")
        if len(path) > 0:
            os.chdir(path)
        pm.execute_notebook(
            notebook_name,
            output_notebook_local_path,
            progress_bar=False,
            request_save_on_cell_execute=True,
            autosave_cell_every=10,
            kernel_name="python3",
            log_output=True,
        )
    finally:
        cancel_interval()
        save_artifacts()
        log("Finished")


if __name__ == "__main__":
    run()



import treebeard.sentry_setup  # type: ignore isort:skip

import warnings
from typing import Any

import click
from halo import Halo  # type: ignore
from humanfriendly import format_size, parse_size  # type: ignore
from timeago import format as timeago_format  # type: ignore

from treebeard.conf import service_status_endpoint
from treebeard.helper import (
    get_config_path,
    get_treebeard_env,
    check_for_updates,
    get_service_status_message,
)
from treebeard.notebooks.commands import cancel, run, status
from treebeard.other.commands import configure, info, version
from treebeard.secrets.commands import secrets

config_path = get_config_path()

treebeard_env = get_treebeard_env()

msg = get_service_status_message(service_status_endpoint)
if msg:
    click.echo(click.style(msg, fg="red"), err=True)

project_id = treebeard_env["project_id"]
notebook_id = treebeard_env["notebook_id"]

warnings.filterwarnings(
    "ignore", "Your application has authenticated using end user credentials"
)


@click.group()
def cli():
    pass


@cli.resultcallback()
def process_result(*args: Any, **kwargs: Any):
    check_for_updates()


cli.add_command(configure)
cli.add_command(status)
cli.add_command(run)
cli.add_command(cancel)
cli.add_command(secrets)
cli.add_command(info)
cli.add_command(version)



import setuptools  # type: ignore

from treebeard.version import get_version

install_requires = [
    "click",
    "google-cloud-storage",
    "halo",
    "papermill",
    "jupyter-repo2docker",
    "colorama",
    "ipykernel",
    "pathlib",
    "timeago",
    "humanfriendly",
    "sentry-sdk",
    "pyyaml",
    "pydantic",
    "docker",
]

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="treebeard",
    version=get_version(),
    author="Treebeard Technologies",
    author_email="alex@treebeard.io",
    description="Tools for notebook hosting",
    long_description=long_description,
    long_description_content_type="text/markdown",  # type: ignore
    packages=setuptools.find_namespace_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
    package_data={"treebeard": ["deploy/*", "treebeard"]},
    entry_points={
        "console_scripts": [
            "treebeard = treebeard.treebeard:cli",
            "treebeard-spaceship = treebeard.version:cli",
        ]
    },
    install_requires=install_requires,
)



import os
import os.path
import tarfile
import tempfile
from typing import IO, List

import click

from treebeard.helper import get_treebeard_config


def get_secrets_archive(files: List[IO] = []) -> IO[bytes]:
    treebeard_config = get_treebeard_config()

    if treebeard_config:
        files += tuple((open(path, "r") for path in treebeard_config.secret))

    for f in files:
        is_in_project = os.path.realpath(f.name).startswith(os.getcwd())
        if not is_in_project:
            click.echo(
                click.style(
                    f"ERROR: {f.name} is not in the notebook directory. All secrets must be located in the notebook directory",
                    fg="red",
                )
            )

    with tempfile.NamedTemporaryFile(
        "wb", suffix=".tar.gz", delete=False
    ) as secrets_archive:
        click.echo("\n")
        with tarfile.open(
            fileobj=secrets_archive, mode="w:gz", dereference=True
        ) as tar:
            for f in files:
                click.echo(f"  Including {f.name}")
                tar.add(f.name)

    return secrets_archive



import configparser
import json
import os
import sys
import time
from pathlib import Path

import click
import requests
from datetime import datetime
from typing import Optional, Tuple
from pydantic import BaseModel
import yaml

from treebeard.version import get_version


if os.getenv("TREEBEARD_ENVIRONMENT") is None:
    env = "production"
else:
    env = os.getenv("TREEBEARD_ENVIRONMENT")


def get_config_path():
    home = str(Path.home())
    return f"{home}/.treebeard"


config_path = get_config_path()


def get_notebook_name():
    treebeard_config = get_treebeard_config()

    notebook = "main.ipynb"
    if treebeard_config:
        notebook = treebeard_config.notebook

    if not os.path.isfile(f"{os.getcwd()}/{notebook}"):
        return None
    return Path(os.getcwd()).name


def get_treebeard_env():
    """Reads variables from a local file, credentials.cfg"""
    config = configparser.RawConfigParser()
    treebeard_project_id = None
    run_id = None
    email = None
    api_key = None

    try:
        config.read(config_path)
        email = config.get("credentials", "TREEBEARD_EMAIL")
        treebeard_project_id = config.get("credentials", "TREEBEARD_PROJECT_ID")
        api_key = config.get("credentials", "TREEBEARD_API_KEY")

        run_id = os.getenv("TREEBEARD_RUN_ID")
        if run_id is None:
            run_id = f"local-{int(time.time())}"
    except:
        pass  # Default values indicate error occurred
    treebeard_env = {
        "notebook_id": get_notebook_name(),
        "project_id": treebeard_project_id,
        "run_id": run_id,
        "email": email,
        "api_key": api_key,
    }
    return treebeard_env


def set_credentials(email: str, key: str, signup_endpoint: str):
    """Create user credentials"""
    # key = secrets.token_urlsafe(16)
    config = configparser.RawConfigParser()
    config.add_section("credentials")
    config.set("credentials", "TREEBEARD_EMAIL", email)
    # Project id is last 10 numbers of hash of email

    click.echo(signup_endpoint)
    response = requests.post(signup_endpoint, headers={"api_key": key, "email": email},)

    if response.status_code != 200:
        raise click.ClickException(f"Request failed: {response.text}")

    try:
        json_data = json.loads(response.text)
        project_id = json_data["project_id"]
        # click.echo(f"✨  Run has been accepted! {json_data['admin_url']}")
    except:
        click.echo("❗  Request to configure failed")
        click.echo(sys.exc_info())
        if response:
            click.echo(response.text)
        return

    config.set("credentials", "TREEBEARD_PROJECT_ID", project_id)
    config.set("credentials", "TREEBEARD_API_KEY", key)
    with open(config_path, "w") as configfile:
        config.write(configfile)
    click.echo(f"🔑  Config saved in {config_path}")
    return project_id


def get_run_path():
    treebeard_env = get_treebeard_env()
    print(f"Treebeard env is {treebeard_env}")
    return f"{treebeard_env['project_id']}/{treebeard_env['notebook_id']}/{treebeard_env['run_id']}"


def get_time():
    return datetime.now().strftime("%H:%M:%S")


def validate_notebook_directory():
    treebeard_env = get_treebeard_env()
    if treebeard_env["project_id"] is None:
        click.echo(
            click.style(
                "This library will not function without credentials.\nPlease email alex@treebeard.io to obtain an API key then run `treebeard configure`",
                fg="red",
            )
        )
        sys.exit(1)
    if treebeard_env["notebook_id"] is None:
        click.echo(
            "Fatal: This command must be run in a directory containing a main.ipynb file or valid treebeard.yaml"
        )
        sys.exit(1)


def check_for_updates():
    version = get_version()

    pypi_data = requests.get("https://pypi.org/pypi/treebeard/json")
    latest_version = json.loads(pypi_data.text)["info"]["version"]

    if latest_version != version:
        click.echo(
            click.style(
                "🌲 Warning: you are not on the latest version of Treebeard, update with `pip install --upgrade treebeard`",
                fg="yellow",
            ),
            err=True,
        )


def get_service_status_message(service_status_url: str) -> Optional[str]:
    data = json.loads(requests.get(service_status_url).text)
    if "message" in data:
        return data["message"]
    return None


class TreebeardConfig(BaseModel):
    notebook: str = "main.ipynb"
    output_dirs: Tuple[str, ...] = tuple(["output"])
    ignore: Tuple[str, ...] = ()
    secret: Tuple[str, ...] = ()


def get_treebeard_config() -> Optional[TreebeardConfig]:
    config = "treebeard.yaml"
    if not os.path.exists(config):
        return None

    with open(config) as f:
        conf = yaml.load(f, Loader=yaml.FullLoader)
        return TreebeardConfig(**conf)
